import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'iroute_stack.dart';
import 'iroute_common.dart';

class IRoute extends RouterDelegate<String>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin {
  GlobalKey<NavigatorState> get navigatorKey =>
      GlobalObjectKey<NavigatorState>(this);

  Map<String, RouteBuilder>? routes;
  //如果指定了child，则表示它不是单独工作的，配合下级
  Widget? child;

  //url历史记录
  IRouteStack stack = IRouteStack(); //stack是单例的，全局只有一个
  IRoute({this.child, this.routes}) {
    stack.root = this;
  }

  static IRoute of(BuildContext context) {
    final delegate = Router.of(context).routerDelegate;
    assert(delegate is IRoute, 'Delegate type must match');
    return delegate as IRoute;
  }

  //如果指定了child
  //如果没有，则pages
  @override
  Widget build(BuildContext context) {
    //print('iroute|build: stack=${stack.routeStack.toString()}');
    return Navigator(
        key: navigatorKey,
        pages: child != null
            ? [
                MaterialPage(
                  child: child!,
                ),
              ]
            : this.stack.getCurrentPages(context, routes!),
        onPopPage: (route, result) {
          if (!route.didPop(result)) {
            return false;
          }
          if (canPop) {
            pop();
          }
          return true;
        });
  }

  @override
  String get currentConfiguration {
    return stack.currentConfiguration!;
  }

  push(String path) {
    //if (path == currentConfiguration) return;
    stack.push(path);
    notifyListeners();
  }

  pop() {
    stack.pop();
    notifyListeners();
  }

  bool get canPop => stack.canPop;
  bool initing = false;

  // @override
  // Future<void> setInitialRoutePath(String configuration) {
  //   initing = true;
  //   return super.setInitialRoutePath(configuration);
  // }

  //setNewPath不需要notifyListeners()
  //注意：两种情况会执行这里
  //首先是setInitialRoutePath之后，我们可设置initing字段为true，而这里将其改为false
  //其次是响应浏览器地址栏变化，包括前进、后退、选历史记录、修改浏览器地址，此时initing=false
  //此时可注意，不向浏览器发送历史记录，如Router.neglect
  @override
  Future<void> setNewRoutePath(String path) async {
    //print('setNewPath:path=$path,stack=${stack.routeStack.toString()}');

    //stack的列表不为空，且path等于当前url，则什么都不做
    if (stack.currentConfiguration != null &&
        path == stack.currentConfiguration) return SynchronousFuture(null);

    stack.setNewRoutePath(path); //加入path，且通知
    return SynchronousFuture(null);
  }
}
