import 'iroute_nested.dart';
import 'page_list.dart';
import 'package:flutter/material.dart';
import 'iroute_common.dart';

class IRouteAppShell extends StatefulWidget {
  IRouteAppShell({required this.pageList, required this.routes});
  final PageList pageList;
  final Map<String, RouteBuilder> routes;

  @override
  IRouteAppShellState createState() =>
      IRouteAppShellState(pageList: pageList, routes: routes);
}

class IRouteAppShellState extends State {
  IRouteAppShellState({required this.pageList, required this.routes})
      : _routerDelegate = IRouteNested(pageList: pageList, routes: routes);

  final PageList pageList;
  final Map<String, RouteBuilder> routes;
  final IRouteNested _routerDelegate;
  ChildBackButtonDispatcher? _backButtonDispatcher;

  @override
  void initState() {
    _routerDelegate.addListener(() => setState(() {}));
    super.initState();
  }

  @override
  void dispose() {
    _routerDelegate..removeListener(() => setState(() {}));
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant IRouteAppShell oldWidget) {
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _backButtonDispatcher = Router.of(context)
        .backButtonDispatcher!
        .createChildBackButtonDispatcher();
  }

  @override
  Widget build(BuildContext context) {
    _backButtonDispatcher!.takePriority();

    List<BottomNavigationBarItem> items = [];
    pageList.pages.forEach((element) {
      items.add(element.bottom);
    });
    //print('AppShell build:selectPaget=${_routerDelegate.selectedPage}');

    return Scaffold(
      body: Router(
        routerDelegate: _routerDelegate,
        backButtonDispatcher: _backButtonDispatcher,
      ),
      bottomNavigationBar: BottomNavigationBar(
        type: BottomNavigationBarType.fixed,
        items: items,
        currentIndex: _routerDelegate.selectedPage,
        onTap: (newIndex) {
          //print('newIndex=$newIndex');
          _routerDelegate.selectedPage = newIndex;
          setState(() {});
          //_routerDelegate.notifyListeners();
        },
      ),
    );
  }
}
