import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'iroute_stack.dart';
import 'url_parser.dart';
import 'page_list.dart';
import 'iroute_common.dart';

class IRouteNested extends RouterDelegate<String>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<String> {
  GlobalKey<NavigatorState> get navigatorKey =>
      GlobalObjectKey<NavigatorState>(this);

  static IRouteNested of(BuildContext context) {
    final delegate = Router.of(context).routerDelegate;
    assert(delegate is IRouteNested, 'Delegate type must match');
    return delegate as IRouteNested;
  }

  IRouteStack stack = IRouteStack(); //stack是单例的，全局只有一个
  late Map<String, RouteBuilder> routes;

  late PageList pageList;

  //error:Null check operator used on a null value
  int get selectedPage {
    return pageList.current;
  }

  set selectedPage(int value) {
    pageList.current = value;
    push(pageList.pages[pageList.current].currentUrl);
    //notifyListeners();
  }

  @override
  void dispose() {
    super.dispose();
    stack.root.removeListener(this.notifyListeners);
  }

  IRouteNested({required this.routes, required this.pageList}) {
    if (stack.root == null) throw ('static should have a root');
    stack.root.addListener(this.notifyListeners);
    if (routes.containsKey('_')) {
      addHome = (context, params) => routes['_']!(context, params);
    }
  }

  RouteBuilder? addHome;

  bool _canPop = true;
  bool get canPop {
    if (_canPop == false) return false;

    return stack.canPop;
  }

  set canPop(bool canPop) => _canPop = canPop;

  Widget _getWidget(context, url) {
    String pattern = UrlParser.getPattern(url);
    var params = UrlParser.getParam(url);

    if (params.containsKey('page'))
      pageList.current = params['page'];
    else
      pageList.current = 0;

    if (pageList.pages[pageList.current].widget == null ||
        pageList.pages[pageList.current].currentUrl != url) {
      if (addHome != null)
        pageList.pages[pageList.current].widget = addHome!(context, {
          'child':
              ((context) => this.routes[pattern]!(context, params))(context)
        });
      else
        pageList.pages[pageList.current].widget =
            ((context) => this.routes[pattern]!(context, params))(context);
    }
    if (pageList.pages[pageList.current].currentUrl != url)
      pageList.pages[pageList.current].currentUrl = url;

    return pageList.pages[pageList.current].widget!;
  }

  Page buildPage(context, url) {
    return MaterialPage(
        key: UniqueKey(), name: url, child: _getWidget(context, url));
  }

  @override
  Widget build(BuildContext context) {
    //if (this._routeStack.isEmpty) _routeStack.add(this.defaultUrl);
    List<Page<dynamic>> _pages = [];
    for (String path in stack.routeStack) _pages.add(buildPage(context, path));
    //print('NestedRoute:routeStack=${stack.routeStack.toString()}');

    return Navigator(
        key: navigatorKey,
        pages: _pages,
        onPopPage: (route, result) {
          if (!route.didPop(result)) {
            return false;
          }
          if (canPop) {
            pop();
          }
          return true;
        });
  }

  @override
  Future<bool> popRoute() {
    //print('Pop Route');
    return super.popRoute();
  }

  void push(String path) {
    if (path == stack.currentConfiguration) return;
    String url = path;

    if (!path.contains(':ipage') && path != '/') {
      url = url + ':ipage=${pageList.current}';
    }

    stack.root.push(url);
    notifyListeners();
  }

  void pop() {
    if (!canPop) return;
    //print('pop in root');
    stack.root.pop(); //这里有重绘，后面的是否会执行？

    var params = UrlParser.getParam(stack.currentConfiguration);
    if (params.containsKey('page'))
      pageList.current = params['page'];
    else
      pageList.current = 0;
    //print('pop in nested,current=${pageList.current}');

    notifyListeners();
  }

  @override
  Future<void> setNewRoutePath(String path) async {
    // This is not required for inner router delegate because it does not
    // parse route
    assert(false);
  }
}
