import 'package:flutter/material.dart';
import 'url_parser.dart';
import 'iroute.dart';
import 'iroute_common.dart';

//单一的路由状态
class IRouteStack {
  //singleton模式，用于处理历史记录者，全局只需要一个
  static final IRouteStack _instance = IRouteStack._internal();
  factory IRouteStack() => _instance;
  IRouteStack._internal();

  //这里留下一个指针，指向唯一的Iroute
  late IRoute _root;
  set root(value) {
    _root = value;
  }

  get root => _root;

  //存储url的历史记录
  List<String> _routeStack = ['/'];
  List<String> get routeStack => _routeStack;

  bool get canPop => _routeStack.length > 1;

  String? get currentConfiguration =>
      _routeStack.isNotEmpty ? _routeStack.last : null;
  void push(String path) {
    if (path == _routeStack.last) return;
    _routeStack.add(path);
  }

  void pop() {
    if (!canPop) return;
    _routeStack.removeLast();
  }

  Future<void> setNewRoutePath(String path) async {
    //这里判断path，若等于当前url，返回
    if (path == currentConfiguration) return;
    _setNewRouteHistory(path);
  }

  void _setNewRouteHistory(String newRoute) {
    List<String> pathsHolder = [];
    pathsHolder.addAll(_routeStack);
    for (String path in _routeStack) {
      if (path == newRoute) {
        int index = _routeStack.indexOf(path);
        int count = _routeStack.length;
        for (var i = index; i < count - 1; i++) {
          pathsHolder.removeLast();
        }
        //这里删掉同名的
        _routeStack.clear();
        _routeStack.addAll(pathsHolder);
        return;
      }
    }
    pathsHolder.add(newRoute);
    _routeStack.clear();
    _routeStack.addAll(pathsHolder);
  }

  WidgetBuilder? getWidget(Map<String, RouteBuilder> routes, url) {
    String pattern = UrlParser.getPattern(url);
    var params = UrlParser.getParam(url);
    if (!routes.containsKey(pattern)) {
      throw '$pattern not exist';
    }
    return (context) => routes[pattern]!(context, params);
  }

  Page buildPage(context, Map<String, RouteBuilder> routes, url) {
    return MaterialPage(
        key: UniqueKey(), name: url, child: getWidget(routes, url)!(context));
  }

  List<Page<dynamic>> getCurrentPages(
      context, Map<String, RouteBuilder> routes) {
    return [
      //if (routeStack.isEmpty) buildPage(context, routes, '/'),
      for (String path in routeStack) buildPage(context, routes, path),
    ];
  }
}
